//---------------------------------------------------------------------------
#include <winsock2.h>
#include <vcl.h>
#pragma hdrstop

#include "SharedCode.h"

#include <memory>
#include <strstream>
#include <fstream>
#include <time.h>

#pragma package(smart_init)

//---------------------------------------------------------------------------
void ShowSimpleMsg(AnsiString asMsg)
{
  MessageBox(Application->Handle,asMsg.c_str(),Application->ExeName.c_str(),MB_OK | MB_TASKMODAL);
}

//--------------------------------------------------------------------
bool WriteToRegistry(HKEY hKeyType, String sKeyName, AnsiString sName,
                     Variant vData, AnsiString sPCName)
{
  bool bSuccess = false;

  try
  {
    if((sKeyName != "") && (sName != ""))
    {
      std::auto_ptr<TRegistry> myReg(new TRegistry());

      if(sPCName != "")
      {
        if (myReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(bSuccess);
        }
      }

      myReg->RootKey = hKeyType;

      if (myReg->OpenKey(sKeyName,True))
      {
        switch (vData.Type())
        {
          case varEmpty:

            break;
          case varNull:

            break;
          case varSmallint:
            myReg->WriteInteger(sName,vData);
            bSuccess = true;
            break;
          case varInteger:
            myReg->WriteInteger(sName,vData);
            bSuccess = true;
            break;
          case varSingle:
            myReg->WriteFloat(sName,vData);
            bSuccess = true;
            break;
          case varDouble:
            myReg->WriteFloat(sName,vData);
            bSuccess = true;
            break;
          case varCurrency:
            myReg->WriteCurrency(sName,vData);
            bSuccess = true;
            break;
          case varDate:
            myReg->WriteDate(sName,vData);
            bSuccess = true;
            break;
          case varOleStr:
            myReg->WriteString(sName,vData);
            bSuccess = true;
            break;
          case varDispatch:

            break;
          case varError:

            break;
          case varBoolean:
            myReg->WriteBool(sName,vData);
            bSuccess = true;
            break;
          case varVariant:
            //myReg->WriteBinaryData(sName,vData,sizeof(vData));
            //bSuccess = true;
            break;
          case varUnknown:

            break;
          case varByte:
            myReg->WriteString(sName,vData);
            bSuccess = true;
            break;
          case varString:
            myReg->WriteString(sName,vData);
            bSuccess = true;
            break;
          case varTypeMask:

            break;
          case varArray:

            break;
          case varByRef:

            break;
          default:

            break;
        }

        //RegFlushKey(hKeyType);
      }


      myReg->CloseKey();
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [method WriteToRegistry()] msg: " + E.Message;
    MessageBox(NULL,asMsg.c_str(),"Problem Detected",MB_OK | MB_TASKMODAL);
  }

  return(bSuccess);
}

//--------------------------------------------------------------------
bool WriteFontToRegistry(HKEY hKeyType, String sKeyName, AnsiString sName,
                         TLogFont *pFont, AnsiString sPCName)
{
  bool bSuccess = false;

  try
  {
    if((sKeyName != "") && (sName != ""))
    {
      std::auto_ptr<TRegistry> myReg(new TRegistry());

      if(sPCName != "")
      {
        if (myReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(bSuccess);
        }
      }

      myReg->RootKey = hKeyType;

      if (myReg->OpenKey(sKeyName,True))
      {
        myReg->WriteBinaryData(sName,pFont,sizeof(TLogFont));
        bSuccess = true;

        //RegFlushKey(hKeyType);
      }

      myReg->CloseKey();
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [method WriteFontToRegistry()] msg: " + E.Message;
    MessageBox(NULL,asMsg.c_str(),"Problem Detected",MB_OK | MB_TASKMODAL);
  }

  return(bSuccess);
}


//--------------------------------------------------------------------
Variant ReadFromRegistry(HKEY hKeyType, AnsiString sKeyName, AnsiString sName,
                         int vType, bool bCheckCurrentUserFirst,
                         AnsiString sPCName)
{
  Variant vData;
  vData = vData.AsType(varNull);

  try
  {
    if((sKeyName != ""))
    {
      std::auto_ptr<TRegistry> pReg(new TRegistry());

      if(sPCName != "")
      {
        if (pReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(NULL);
        }
      }

      bool bDone = false;
      while(bDone == false)
      {
        if(bCheckCurrentUserFirst == true)
        {
          pReg->RootKey = HKEY_CURRENT_USER;
        }
        else
        {
          pReg->RootKey = hKeyType;
        }

        bool bFound = true;

        if(pReg->OpenKey(sKeyName,false))
        {
          if(pReg->ValueExists(sName) == true || sName == "")
          {
            switch (vType)
            {
              case varEmpty:

                break;
              case varNull:

                break;
              case varSmallint:
                vData = pReg->ReadInteger(sName);
                break;
              case varInteger:
                vData = pReg->ReadInteger(sName);
                break;
              case varSingle:
                vData = pReg->ReadFloat(sName);
                break;
              case varDouble:
                vData = pReg->ReadFloat(sName);
                break;
              case varCurrency:
                vData = pReg->ReadCurrency(sName);
                break;
              case varDate:
                vData = pReg->ReadDate(sName);
                break;
              case varOleStr:
                vData = pReg->ReadString(sName);
                break;
              case varDispatch:

                break;
              case varError:

                break;
              case varBoolean:
                vData = pReg->ReadBool(sName);
                break;
              case varVariant:
                //pReg->ReadBinaryData(sName,vData,sizeof(vData));
                break;
              case varUnknown:

                break;
              case varByte:
                vData = pReg->ReadString(sName);
                break;
              case varString:
                if(pReg->ReadString(sName) != "")
                {
                  vData = pReg->ReadString(sName);
                }

                break;
              case varTypeMask:

                break;
              case varArray:

                break;
              case varByRef:

                break;
              default:

                break;
            }
          }
          else
          {
            bFound = false;
          }
        }
        else
        {
          bFound = false;
        }

        if(bFound == false && bCheckCurrentUserFirst == true)
        {
          bCheckCurrentUserFirst = false;
        }
        else
        {
          bDone = true;
        }

        pReg->CloseKey();
      }
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [ReadFromRegistry()] msg: " + E.Message;
    MessageBox(NULL,asMsg.c_str(),"Problem Detected",MB_OK | MB_TASKMODAL);
  }

  return(vData);
}

//--------------------------------------------------------------------
TLogFont * ReadFontFromRegistry(HKEY hKeyType, AnsiString sKeyName, AnsiString sName,
                                bool bCheckCurrentUserFirst, AnsiString sPCName)
{
  TLogFont *pFont=NULL;

  try
  {
    if((sKeyName != ""))
    {
      std::auto_ptr<TRegistry> pReg(new TRegistry());

      if(sPCName != "")
      {
        if (pReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(pFont);
        }
      }

      bool bDone = false;
      while(bDone == false)
      {
        if(bCheckCurrentUserFirst == true)
        {
          pReg->RootKey = HKEY_CURRENT_USER;
        }
        else
        {
          pReg->RootKey = hKeyType;
        }

        bool bFound = true;

        if(pReg->OpenKey(sKeyName,false))
        {
          if(pReg->ValueExists(sName) == true || sName == "")
          {
            pFont = new TLogFont;
            pReg->ReadBinaryData(sName,pFont,sizeof(TLogFont));
          }
          else
          {
            bFound = false;
          }
        }
        else
        {
          bFound = false;
        }

        if(bFound == false && bCheckCurrentUserFirst == true)
        {
          bCheckCurrentUserFirst = false;
        }
        else
        {
          bDone = true;
        }

        pReg->CloseKey();
      }
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [ReadFromRegistry()] msg: " + E.Message;
    MessageBox(NULL,asMsg.c_str(),"Problem Detected",MB_OK | MB_TASKMODAL);
  }

  return(pFont);
}


//--------------------------------------------------------------------
TStringList * ReadKeysFromRegistry(HKEY hKeyType, AnsiString sKeyName,
                                   bool bCheckCurrentUserFirst, AnsiString sPCName)
{
  TStringList *pList = new TStringList();

  try
  {
    if(sKeyName != "")
    {
      std::auto_ptr<TRegistry> pReg(new TRegistry());

      if(sPCName != "")
      {
        if (pReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(pList);
        }
      }

      bool bDone = false;
      while(bDone == false)
      {
        if(bCheckCurrentUserFirst == true)
        {
          pReg->RootKey = HKEY_CURRENT_USER;
        }
        else
        {
          pReg->RootKey = hKeyType;
        }

        bool bFound = true;

        if(pReg->OpenKey(sKeyName,false))
        {
          pReg->GetKeyNames(pList);
        }
        else
        {
          bFound = false;
        }

        if(bFound == false && bCheckCurrentUserFirst == true)
        {
          bCheckCurrentUserFirst = false;
        }
        else
        {
          bDone = true;
        }

        pReg->CloseKey();
      }
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [ReadKeysFromRegistry()] msg: " + E.Message;
    MessageBox(NULL,asMsg.c_str(),"Problem Detected",MB_OK | MB_TASKMODAL);
  }

  return(pList);
}

//--------------------------------------------------------------------
TStringList * ReadKeyValueNamesFromRegistry(HKEY hKeyType, AnsiString sKeyName,
                                   bool bCheckCurrentUserFirst, AnsiString sPCName)
{
  TStringList *pList = new TStringList();

  try
  {
    if(sKeyName != "")
    {
      std::auto_ptr<TRegistry> pReg(new TRegistry());

      if(sPCName != "")
      {
        if (pReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(pList);
        }
      }

      bool bDone = false;
      while(bDone == false)
      {
        if(bCheckCurrentUserFirst == true)
        {
          pReg->RootKey = HKEY_CURRENT_USER;
        }
        else
        {
          pReg->RootKey = hKeyType;
        }

        bool bFound = true;

        if(pReg->OpenKey(sKeyName,false))
        {
          pReg->GetValueNames(pList);
        }
        else
        {
          bFound = false;
        }

        if(bFound == false && bCheckCurrentUserFirst == true)
        {
          bCheckCurrentUserFirst = false;
        }
        else
        {
          bDone = true;
        }

        pReg->CloseKey();
      }
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [ReadKeyValueNamesFromRegistry()] msg: " + E.Message;
    MessageBox(NULL,asMsg.c_str(),"Problem Detected",MB_OK | MB_TASKMODAL);
  }

  return(pList);
}


//--------------------------------------------------------------------
bool DeleteKeyFromRegistry(HKEY hKeyType, AnsiString sKeyName, bool bCheckCurrentUserFirst,
                           AnsiString sPCName)
{
  bool bFound = false;

  try
  {
    if((sKeyName != ""))
    {
      std::auto_ptr<TRegistry> pReg(new TRegistry());

      if(sPCName != "")
      {
        if (pReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(NULL);
        }
      }

      bool bDone = false;
      while(bDone == false)
      {
        if(bCheckCurrentUserFirst == true)
        {
          pReg->RootKey = HKEY_CURRENT_USER;
        }
        else
        {
          pReg->RootKey = hKeyType;
        }

        bFound = pReg->DeleteKey(sKeyName);

        if(bFound == false && bCheckCurrentUserFirst == true)
        {
          bCheckCurrentUserFirst = false;
        }
        else
        {
          bDone = true;
        }

        pReg->CloseKey();
      }
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [DeleteKeyFromRegistry()] msg: " + E.Message;
    Application->BringToFront();
    MessageBox(NULL,asMsg.c_str(),"Problem Detected",MB_OK | MB_TASKMODAL);
  }

  return(bFound);
}

//--------------------------------------------------------------------
bool DeleteFromRegistry(HKEY hKeyType, AnsiString sKeyName, AnsiString sName,
                        bool bCheckCurrentUserFirst,  AnsiString sPCName)
{
  bool bFound = false;

  try
  {
    if((sKeyName != ""))
    {
      std::auto_ptr<TRegistry> pReg(new TRegistry());

      if(sPCName != "")
      {
        if (pReg->RegistryConnect("\\\\" + sPCName) == false)
        {
          return(NULL);
        }
      }

      bool bDone = false;
      while(bDone == false)
      {
        if(bCheckCurrentUserFirst == true)
        {
          pReg->RootKey = HKEY_CURRENT_USER;
        }
        else
        {
          pReg->RootKey = hKeyType;
        }

        if(pReg->OpenKey(sKeyName,false))
        {
          if(pReg->ValueExists(sName) == true || sName == "")
          {
            bFound = pReg->DeleteValue(sName);
          }
        }
        else
        {
          bFound = false;
        }

        if(bFound == false && bCheckCurrentUserFirst == true)
        {
          bCheckCurrentUserFirst = false;
        }
        else
        {
          bDone = true;
        }

        pReg->CloseKey();
      }
    }
  }
  catch(Exception &E)
  {
    AnsiString asMsg = "Internal Error in [DeleteFromRegistry()] msg: " + E.Message;
    MessageBox(NULL,asMsg.c_str(),"Problem Detected",MB_OK | MB_TASKMODAL);
  }

  return(bFound);
}

// Launch an external process
int StartProcess(AnsiString sAppName, WORD wShowWindow,
                 bool bWaitUntilDone, bool bReturnExitCode,
                 bool bReturnProcessID)
{
  int   iRetVal = 0;
  STARTUPINFO  StartupInfo;
  ZeroMemory( &StartupInfo, sizeof(STARTUPINFO));

  StartupInfo.cb = sizeof(STARTUPINFO);
  StartupInfo.dwFlags = STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow = wShowWindow;

  PROCESS_INFORMATION ProcessInfo;

  AnsiString asLaunch = sAppName;

  if(CreateProcess(NULL, asLaunch.c_str(), NULL, NULL, TRUE, NORMAL_PRIORITY_CLASS,
                   NULL, NULL, &StartupInfo, &ProcessInfo))
  {
      if(bWaitUntilDone == true)
      {
        while(WaitForSingleObject(ProcessInfo.hProcess,50) != WAIT_OBJECT_0)
        {
          if(Screen->ActiveForm)
          {
            Screen->ActiveForm->Update();
          }
        }
      }

      if(bReturnExitCode)
      {
        DWORD dwExitCode = 0;

        bool bSuccess = GetExitCodeProcess(ProcessInfo.hProcess,&dwExitCode);

        iRetVal = dwExitCode;
      }
      else if(bReturnProcessID)
      {
        iRetVal = ProcessInfo.dwProcessId;
      }
      else
      {
        iRetVal = 1;
      }
  }

  // We must close the handles returned in ProcessInfo. We can
  // close the handle at any time, might as well close it now
  CloseHandle(ProcessInfo.hProcess);
  CloseHandle(ProcessInfo.hThread);

  return(iRetVal);
}

///in source
String __fastcall VersionInfo(const String &sQuery, AnsiString asFile)
{
    DWORD c;
    DWORD dw=0;
    UINT ui;
    char *p;
    LPVOID ptr;
    String sOut=(asFile != "" ? asFile : Application->ExeName); //ParamStr(0); ///ParamStr(0) holds exe name

    c = GetFileVersionInfoSize(sOut.c_str()/*ParamStr(0).c_str()*/, &dw);
    p = new char[c + 1]; //file://create the space
    GetFileVersionInfo(sOut.c_str()/*ParamStr(0).c_str()*/,0,c,p);//get the version info data

/// Extract the language/translation information...
    VerQueryValue(p, TEXT("\\VarFileInfo\\Translation"), &ptr, &ui);

/// ptr comes back as a ptr to two (16-bit) words containing the two halves of
/// the translation number required for StringFileInfo
    WORD *id=(WORD*)ptr;
    String sBase = String(  "\\StringFileInfo\\")+
                            IntToHex(id[0],4) +
                            IntToHex(id[1],4) +
                            "\\";
    String qs = sBase + sQuery ; // query string
    VerQueryValue(p, qs.c_str(), &ptr, &ui);
    sOut = (char*)ptr;
    delete [] p;
    return sOut;
}

//----------------------------------------------------------------------
std::vector<AnsiString> GetDelimitedValues(AnsiString asData,AnsiString asDelimiter)
{
  std::vector<AnsiString> vctParsedData;
  vctParsedData.clear();

  int iPos   = asData.AnsiPos(asDelimiter);
  bool bDone = !iPos;
  while(bDone == false)
  {
    AnsiString asItem = asData.SubString(1,iPos-1);
    vctParsedData.push_back(asItem);

    asData.Delete(1,iPos);
    iPos = asData.AnsiPos(asDelimiter);
    bDone = !iPos;

    if(iPos <= 0 && asData.Length() > 0)
    {
      // This will ensure we get the last NON-delimited item
      vctParsedData.push_back(asData);

      bDone = true;
    }
    else if(iPos <= 0)
    {
      // This will ensure we get the last NON-delimited EMPTY value
      vctParsedData.push_back("");

      bDone = true;
    }
  }

  return(vctParsedData);
}

//----------------------------------------------------------------------
bool __fastcall StartNewBrowser(AnsiString asURL, AnsiString asLocalHTMLFile, int iShowCmd)
{
   bool bRetVal = false;
   // start a new instance of the user's browser
   // at the page passed as sURL
   long success  = 0;
   long hProcess = 0;

   STARTUPINFO  StartupInfo;
   ZeroMemory( &StartupInfo, sizeof(STARTUPINFO));

   StartupInfo.cb = sizeof(STARTUPINFO);
   StartupInfo.dwFlags = STARTF_USESHOWWINDOW;
   StartupInfo.wShowWindow = iShowCmd;

   PROCESS_INFORMATION proc;

   AnsiString asBrowser = GetApplicationName(success, asLocalHTMLFile);

   // did sBrowser get correctly filled?
   if(success >= 32)
   {
      //start a new instance of the default
      //browser at the specified URL. The
      //lpCommandLine member (second parameter)
      //requires a leading space or the call
      //will fail to open the specified page.
      asURL = " " + asURL;

      success = CreateProcess(asBrowser.c_str(),asURL.c_str(),
                              NULL, NULL, TRUE, NORMAL_PRIORITY_CLASS,
                              NULL, NULL, &StartupInfo, &proc);

      //if the process handle is valid, return success
      bRetVal = (proc.hProcess != 0);

      //don't need the process
      //handle anymore, so close it
      CloseHandle(proc.hProcess);

      //and close the handle to the thread created
      CloseHandle(proc.hThread);
   }

   // Last resort, try ShellExecute
   if(bRetVal == false)
   {
     HINSTANCE hRetVal = ShellExecute(NULL, "open", asURL.c_str(), NULL,NULL,SW_SHOWDEFAULT);
     if((int)hRetVal > 32)
     {
       bRetVal = true;
     }
   }

   return(bRetVal);
}

//----------------------------------------------------------------------
AnsiString __fastcall GetApplicationName(long &dwFlagReturned, AnsiString asHTMLFile)
{
  //find the full path and name of the user's
  //associated browser
  long hFile = 0;
  char szBuf[MAX_PATH+1]="";

  //get the file path & name associated with the file
  dwFlagReturned = (long)FindExecutable(asHTMLFile.c_str(), ExtractFilePath(asHTMLFile).c_str(), szBuf);

  return(szBuf);
}

//----------------------------------------------------------------------
bool __fastcall CreateNewFile(AnsiString asFile)
{
  AnsiString asAppDir = ExtractFilePath(Application->ExeName);

  AnsiString asNewFile = asAppDir + asFile;

  HANDLE hFile = CreateFile(asNewFile.c_str(), 0, FILE_SHARE_READ | FILE_SHARE_WRITE,
                            NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

  bool bRetval = (hFile != INVALID_HANDLE_VALUE);
  return(bRetval);
}

//----------------------------------------------------------------------
bool __fastcall RemoveFile(AnsiString asFile)
{
  AnsiString asAppDir = ExtractFilePath(Application->ExeName);
  AnsiString asNewFile = asAppDir + asFile;

  bool bRetval = DeleteFile(asNewFile.c_str());
  return(bRetval);
}

//---------------------------------------------------------------------------
/* code to establish a socket; originally from bzs@bu-cs.bu.edu  */
int __fastcall ConnectToHost(char *myname, unsigned short portnum, int &iSocketErr,int iSocketType)
{
  int     iSocket;
  struct  sockaddr_in     sin;
  struct  sockaddr_in     myaddr;
  struct  servent         *sp;
  struct  hostent         *hp;

  memset((char *)&sin, 0, sizeof(sin));
  memset(&myaddr, 0, sizeof(myaddr));

  try
  {
    sin.sin_port = htons(portnum);

    if((hp = gethostbyname(myname)) == NULL)
    {
      iSocketErr = WSAGetLastError();
      throw EAbort("");
    }

    // Our IP address
    //struct in_addr *pIP = (struct in_addr *)*hp->h_addr_list;

    sin.sin_family = hp->h_addrtype;
    memmove(&sin.sin_addr, hp->h_addr_list[0], hp->h_length);

    if((iSocket = socket(PF_INET,iSocketType,0)) < 0)
    {
      iSocketErr = WSAGetLastError();
      throw EAbort("");
    }

    myaddr.sin_family = AF_INET;
    myaddr.sin_addr.s_addr = htonl (INADDR_ANY);

    if (bind(iSocket, (struct sockaddr *)&myaddr, sizeof(myaddr)) < 0)
    {
      iSocketErr = WSAGetLastError();
      closesocket(iSocket);
      throw EAbort("");
    }

    if (connect(iSocket, (struct sockaddr *)&sin,sizeof(sin)) < 0)
    {
      iSocketErr = WSAGetLastError();
      closesocket(iSocket);
      throw EAbort("");
    }
  }
  catch(const char *ex)
  {
    return(-1);
  }
  catch(const Exception &ex)
  {
    return(-1);
  }
  catch(...)
  {
    AnsiString asMsg = "UNKNOWN Internal error in [ConnectToHost]";
    MessageBox(NULL,asMsg.c_str(),"Error",MB_OK | MB_TASKMODAL);
  }

  return(iSocket);
}

//---------------------------------------------------------------------------
/* code to establish a socket; originally from bzs@bu-cs.bu.edu  */
bool __fastcall CanConnectToHost(AnsiString asHostName, unsigned short portnum,
                                 AnsiString &asErrMsg)
{
  bool bRetVal   = false;
  int iSocket    = -1;
  int iSocketErr = -1;

  if((iSocket = ConnectToHost(asHostName.c_str(), portnum, iSocketErr)) < 0) // plug in the phone
  {
    const char *pError = WSAGetLastErrorMessage("Calling ConnectToHost()",iSocketErr);
    asErrMsg           = "Could not connect to: " + asHostName +
                         " on port: " + portnum + " msg: " + AnsiString(pError);
    bRetVal            = false;
  }
  else
  {
    closesocket(iSocket);
    asErrMsg = "";
    bRetVal = true;
  }

  return(bRetVal);
}

//---------------------------------------------------------------------------
bool __fastcall FormExists(AnsiString asName)
{
  for(int i = 0; i < Screen->FormCount; i++)
  {
    if(Screen->Forms[i]->Name == asName)
    {
      return(true);
    }
  }

  return(false);
}

//---------------------------------------------------------------------------
TForm * __fastcall FindForm(AnsiString asName)
{
  for(int i = 0; i < Screen->FormCount; i++)
  {
    if(Screen->Forms[i]->Name == asName)
    {
      return(Screen->Forms[i]);
    }
  }

  return(NULL);
}

//---------------------------------------------------------------------------
AnsiString __fastcall GetCommaTextFromMap(std::map<AnsiString,bool> &mapList, bool bQuotedTextItems)
{
  AnsiString asList = "";
  AnsiString asQuotedText = "";
  if(bQuotedTextItems == true)
  {
    asQuotedText = "'";
  }

  for(std::map<AnsiString,bool>::iterator iterMap = mapList.begin();
      iterMap != mapList.end(); iterMap++)
  {
    if(iterMap->second)
    {
      if(asList != "")
      {
        asList += ",";
      }

      asList += (asQuotedText + iterMap->first + asQuotedText);
    }
  }

  return(asList);
}

//---------------------------------------------------------------------------
AnsiString __fastcall GetCommaTextFromVector(std::vector<int> &vctList, bool bQuotedTextItems)
{
  AnsiString asRet = "";

  AnsiString asQuotedText = "";
  if(bQuotedTextItems == true)
  {
    asQuotedText = "'";
  }

  int iSize = vctList.size();
  for(int i = 0; i < iSize; i++)
  {
    if(i > 0)
    {
      asRet += ",";
    }

    asRet += (asQuotedText + IntToStr(vctList[i]) + asQuotedText);
  }

  return(asRet);
}

//---------------------------------------------------------------------------
std::string StripCRLF(std::string sIn)
{
  std::string sTmp = sIn;

  // First remove all CR (carriage returns)
  int iPos = sTmp.find("\r");
  while(iPos >= 0 && iPos <= sTmp.length())
  {
    sTmp.erase(iPos,1);
    iPos = sTmp.find("\r");
  }

  // Next remove all LF (Line feeds AKA new line)
  iPos = sTmp.find("\n");
  while(iPos >= 0 && iPos <= sTmp.length())
  {
    sTmp.erase(iPos,1);
    iPos = sTmp.find("\n");
  }

  return(sTmp);
}

//---------------------------------------------------------------------------
void StripSpaces(AnsiString &sIn)
{
  int iPos = sIn.AnsiPos(" ");
  while(iPos > 0)
  {
    sIn.Delete(iPos,1);
    iPos = sIn.AnsiPos(" ");
  }
}

//---------------------------------------------------------------------------
void __fastcall WriteTextToDC(HWND WinHandle, AnsiString asText, int X, int Y)
{
  HDC DC = GetWindowDC(WinHandle);
  if(DC)
  {
    ExtTextOut(DC, X, Y, ETO_CLIPPED, 0, asText.c_str(), asText.Length(), 0);
    ReleaseDC(WinHandle, DC);
  }
}

bool __fastcall LaunchInDefaultViewer(HWND hHandle, AnsiString asFile)
{
  HINSTANCE hRetVal = ShellExecute(hHandle, "open", asFile.c_str(), NULL,NULL,SW_SHOWDEFAULT);
  if((int)hRetVal <= 32)
  {
    //ShowSimpleMsg("Error Launching your Web Browser");
    return(false);
  }

  return(true);
}

//---------------------------------------------------------------------------
void __fastcall LogDebugMsg(AnsiString asLOGFILE,AnsiString asMsg, bool bCreateIfMissing)
{
  try
  {
    // to turn OFF logging, delete the log file
    AnsiString asNewLog = asLOGFILE;
    if (FileExists(asNewLog) == false)
    {
      if(bCreateIfMissing)
      {
        if(CreateNewFile(asNewLog) == false)
        {
          throw Exception("Cannot create LOGFILE: " + asNewLog + " for msg: " + asMsg + " asLOGFILE: " + asLOGFILE);
        }
      }
      else
      {
        return;
      }
    }

    bool bDoneWrite = false;
    for(int i = 1; i <= 3 && bDoneWrite == false; i++)
    {
      try
      {
        // PIECE TOGETHER A STANDARD LOOK FOR EACH LOG ENTRY
        ::time_t now;
        time(&now);

        std::ostrstream osDbg;
        osDbg.seekp(0);
        osDbg << "PID: " << GetCurrentProcessId() << " THREAD ID: " << GetCurrentThreadId() << " " << asMsg.c_str() << " " << ctime(&now) << std::endl << '\0';

        // Setup the file output stuff
        std::ofstream of;
        of.open(asNewLog.c_str(),std::ios_base::app);
        of << osDbg.str();
        of.flush();
        of.close();

        bDoneWrite = true;
      }
      catch(const char *ex)
      {
        std::string err = "Internal Error in [WriteToLog] msg: " + std::string(ex);

        ::Sleep(100); // Pause for retry

        //continue;
      }
      catch(...)
      {
        std::string err = "UNKNOWN Internal Error in [OutputDebug]";

        ::Sleep(100); // Pause for 1 second and retry

        //continue;
      }
    }
  }
  catch(const Exception &ex)
  {
    AnsiString asMsg = "Internal error in [LogDebugMsg] msg: " + ex.Message;
    MessageBox(NULL,asMsg.c_str(),"ERROR",MB_OK | MB_TASKMODAL);
  }
  catch(...)
  {
    AnsiString asMsg = "UNKNOWN Internal error in [LogDebugMsg]";
    MessageBox(NULL,asMsg.c_str(),"ERROR",MB_OK | MB_TASKMODAL);
  }
}

std::vector<AnsiString> GetLocalIPAddressListAndHostname(AnsiString *asHostname, AnsiString *asIPList)
{
  std::vector<AnsiString> vctAddrHostList;

  std::auto_ptr<WSADATA> lpwsaData_m;
  WORD wVersionRequested = MAKEWORD(1, 1);

  /* Use winsock function */
  lpwsaData_m.reset(new WSADATA);
  int nResult = WSAStartup(wVersionRequested, lpwsaData_m.get());
  if(nResult != 0)
  {
    throw Exception("Problem Initializing Winsock!");
  }
  else if(LOBYTE(lpwsaData_m->wVersion) != 1 || HIBYTE(lpwsaData_m->wVersion) != 1)
  {
    throw Exception("Problem with Winsock Version: " + IntToStr(LOBYTE(lpwsaData_m->wVersion)) + "." + IntToStr(HIBYTE(lpwsaData_m->wVersion)));
  }

  struct sockaddr_in sa, isa;

  memset(&sa, 0, sizeof(sa));
  memset(&isa, 0, sizeof(isa));

  char hostname[255] = "";
  int iRet = gethostname(hostname, sizeof(hostname));
  if(iRet == 0)
  {
    if(asHostname)
    {
      *asHostname = hostname;
    }

    struct hostent *hp = gethostbyname(hostname);
    if(hp != NULL)
    {
      for(int i = 0; hp->h_addr_list[i] != NULL; i++)
      {
        // inet_ntoa converts the ints to a string also.
        AnsiString IPAddress = IntToStr((int)(unsigned char)hp->h_addr_list[i][0]) +
                               "." + IntToStr((int)(unsigned char)hp->h_addr_list[i][1]) +
                               "." + IntToStr((int)(unsigned char)hp->h_addr_list[i][2]) +
                               "." + IntToStr((int)(unsigned char)hp->h_addr_list[i][3]);

        vctAddrHostList.push_back(IPAddress);

        if(asIPList)
        {
          if(*asIPList != "")
          {
            (*asIPList) += ",";
          }

          (*asIPList) += IPAddress;
        }
      }
    }
  }

  WSACleanup();

  return(vctAddrHostList);
}


//---------------------------------------------------------------------------
