/**
 * rijndael-alg-fst.h
 *
 * @version 3.0 (December 2000)
 *
 * Optimised ANSI C code for the Rijndael cipher (now AES)
 *
 * @author Vincent Rijmen <vincent.rijmen@esat.kuleuven.ac.be>
 * @author Antoon Bosselaers <antoon.bosselaers@esat.kuleuven.ac.be>
 * @author Paulo Barreto <paulo.barreto@terra.com.br>
 *
 * This code is hereby placed in the public domain.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __RIJNDAEL_ALG_FST_H
#define __RIJNDAEL_ALG_FST_H

//#include "vsMemoryManagement.h"

#ifdef __WIN32__

  #ifndef BYTE
  //typedef unsigned char BYTE;
  #include "wtypes.h"
  typedef byte BYTE;

  #endif

#else

  #ifndef BYTE
  //typedef unsigned char BYTE;
  typedef char BYTE;
  #endif

#endif

#define MAXKC	(256/32)
#define MAXKB	(256/8)
#define MAXNR	1024

typedef unsigned char	u8;	
typedef unsigned short	u16;	
typedef unsigned int	u32;

#ifdef __BORLANDC__

#include <system.hpp>

#endif

PACKAGE int rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBits);
PACKAGE int rijndaelKeySetupDec(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBits);
PACKAGE void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 ct[16]);
PACKAGE void rijndaelDecrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 ct[16], u8 pt[16]);

PACKAGE char *EncryptBuffer(char *pCipherKey, const unsigned int iCipherLen, const char *pBuf, unsigned int &iBufLen);
PACKAGE char *DecryptBuffer(char *pCipherKey, const unsigned int iCipherLen, const char *pBuf, unsigned int &iBufLen);

PACKAGE char * EncryptedBufferToHexText(const char *pBuf,unsigned int &iBufLen);
PACKAGE char * HexTextToEncryptedBuffer(const char *pBuf,unsigned int &iBufLen);

#ifdef INTERMEDIATE_VALUE_KAT
PACKAGE void rijndaelEncryptRound(const u32 rk[/*4*(Nr + 1)*/], int Nr, u8 block[16], int rounds);
PACKAGE void rijndaelDecryptRound(const u32 rk[/*4*(Nr + 1)*/], int Nr, u8 block[16], int rounds);
#endif /* INTERMEDIATE_VALUE_KAT */

#endif /* __RIJNDAEL_ALG_FST_H */
